package org.gcube.accounting.datamodel.validators;

import java.io.Serializable;
import java.util.Map;

import org.gcube.accounting.datamodel.annotations.NotEmptyIfNotNull;
import org.gcube.accounting.exception.InvalidValueException;

public class NotEmptyIfNotNullValidator implements FieldValidator<NotEmptyIfNotNull>{

	public Class<NotEmptyIfNotNull> annotation() {
		return NotEmptyIfNotNull.class;
	}

	public boolean isValid(Serializable toValidate) {
		if (toValidate == null) return true;
		if (toValidate.getClass().isArray() ){
			return ((Object[])toValidate).length>0;
		}else if ( toValidate instanceof Iterable<?>){
			return ((Iterable<?>) toValidate).iterator().hasNext();
		} else if (toValidate instanceof Map<?,?>){
			return ((Map<?,?>) toValidate).size()>0;
		} else if (toValidate instanceof String ){
			return !((String)toValidate).isEmpty();
		} else return true;
	}
	
	public Serializable validate(Serializable toValidate) throws InvalidValueException {
		try{
			if(isValid(toValidate)){
				return toValidate;
			}
		}catch(Exception e){
			throw new InvalidValueException(getErrorSuffix(), e.getCause());
		}
		throw new InvalidValueException(getErrorSuffix());
	}

	public String getErrorSuffix() {
		return "is empty";
	}

}
